function MasterScriptWithFeatCalculation
% Run the analysis and provide supporing data for the unmixing paper
%
% This script starts from raw images and does feature calculation, all
% analysis, and figure generation
%

% Author: R.F. Murphy
% Created: 11-Jan-2010
% Last Update: 17-Jan-2010
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


%% check whether the intermediate results files are present
resultpath = ['./inter_results'];
if exist(resultpath,'dir')
    error('The intermediate results directory (./inter_results) was found - please delete this directory if you wish to restart from feature calculation.');
end

%% Initialize paths
addpath(genpath('./functions'));
addpath(genpath('./functions/SLIC'));
addpath(genpath('./functions/tztoolbox'));
addpath(genpath('./functions/externalFunctions'));
YesFeatureReady = false;

%% Plate unmixing experiment
gnf_unmixMitoLyso(2,YesFeatureReady);

%% Outlier removal analysis
gnf_objoutlieranalysis(YesFeatureReady);
rmdir('./inter_results/T1/tmp','s')
gnf_patternoutlieranalysis(YesFeatureReady);
rmdir('./inter_results/T2/tmp','s')

%% Drug data: relocation analysis
gnf_drugefftanalysis(YesFeatureReady)

%% Remove non-discriminative object type analysis
gnf_loocvrmobjtypes
gnf_rmsimobjalys(5,'./inter_results/features/MitoLyso/prot',2);

%% Generate figures
gnf_genfigs(blockIndex);
