function example_objects_median(block)
% find the "median object" for each type of object and show it
%
% Author: R.F. Murphy
% Created: 22-Jun-2008
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

resultpath = ['./inter_results/block' int2str(block)];

% sort the object types by increasing preference for class 2 over class 1
load([resultpath '/objectdistri.mat'])
objhist = squeeze(mean(objnumdistr));
[sorted,idx] = sort(objhist(:,2)-objhist(:,1));

load([resultpath '/unmixtrain'])
clear objnumdistr
clear objhist

load([resultpath '/objlearned.mat'])
objtype = objectlearned.objtype;
allfeat = objectlearned.allfeat;
zallfeat = objectlearned.zscoredfeat;
best_ncluster = objectlearned.best_ncluster;

% the image id's in "imid" are within a given class, so need to know which
% class an image is from in order to interpret its id
startclass2 = find(classid==2);
startclass2 = startclass2(1);
for i = 1:best_ncluster
    % find all object indices for a given object type
    objind = find(objtype==idx(i));
    % get the features for just those objects
    objfeatx = zallfeat(objind,:);
    % find the one closest to the mean
    meanobjfeat = mean(objfeatx,1);
    sqdistobj = sum((objfeatx-repmat(meanobjfeat,size(objfeatx,1),1)).^2,2);
    [dxx,ismalld] = min(sqdistobj);
    medianobject = objind(ismalld);
%    objtype(medianobject)
    % get the features of that object
    objfeatsfromallfeat(:,i) = allfeat(medianobject,:);
%    objfeatsfromallfeat(:,i)'
    % find the image id that contains that object
    imageid(i) = imid(medianobject);
    % find the number of the object within that image
    objindexwithinimage(i) = ojid(medianobject);
    % find out the class of the image from which that object came
    classofimage(i) = classid(medianobject);
%end    

fpath=['./inter_results/features/MitoLyso/prot'];
% ipath = '/Volumes/Murphy320GB/Images/GNF images/lyso mito unmixing set 1/data/images';
ipath = './images/MitoLyso';
% block=2;
%for i=1:best_ncluster
    % find out which well this image is from
    t=floor(imageid(i)/100);
    k=imageid(i)-t*100;
    if classofimage(i)==1
        findex1=8;
        findex2=t;
    else
        findex1=t;
        findex2=8;
    end
    name = [fpath '/well' num2str(block) num2str(findex1) num2str(findex2) '/feat'...
            num2str(k) '.mat'];
    disp(['Object-type ' num2str(i) ': ' name])
    load(name)        

    % features for that object according to saved .mat file
    precalc = objfeat(objindexwithinimage(i),:)'; 
    % check that the features match
    if max(abs(precalc-objfeatsfromallfeat(:,i)))>0 error('object feature mismatch.'); end
    % now get the coordinates and gray levels of pixels in that object
    coords=gnf_objcoords(ipath,[findex1,findex2,block],2*k-1);
    coordskeep{i}=coords{objindexwithinimage(i)};
    sizekeep(1,i)=max(coordskeep{i}(:,1))-min(coordskeep{i}(:,1))+1;
    sizekeep(2,i)=max(coordskeep{i}(:,2))-min(coordskeep{i}(:,2))+1;
end

objcfile = [resultpath '/objcoordinfo.mat'];
save(objcfile, 'objfeatsfromallfeat', 'imageid', 'objindexwithinimage', 'classofimage', ...
    'coordskeep', 'sizekeep', 'best_ncluster');

% choose dimensions for figure to fit all objects
nwide=ceil(sqrt(best_ncluster));
nhigh=floor((best_ncluster+nwide-1)/nwide);
iwide=max(sizekeep(1,:)); ihigh=max(sizekeep(2,:));
% to turn off constant size image, set iwide and ihigh to 0
for i=1:best_ncluster
    subplot(nwide,nhigh,i)
    show_object(coordskeep{i}(:,1),coordskeep{i}(:,2),coordskeep{i}(:,3),iwide,ihigh)
end
