function [objfeat,objfluo,num_dna] = gnf_DNAimfeat(dnaimg)
% GNF_DNAIMFEAT calculates DNA object features of all cells within an image
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% Load images and preprocess
dna = ml_preprocess(double(dnaimg),[],'ml','yesbgsub');
if isempty(dna)
    objfeat = [];
    objfluo = [];
    num_dna = 0;
    return
end

% Calculate features
dnamask = imopen(dna>0,strel('disk',5));
dnamask = imfill(dnamask,'holes');
[dnalabeled,num_dna] = bwlabel(dnamask,4);
if num_dna <= 8
    impurity = imdilate(dnalabeled>0,strel('disk',10));
    dnaimg(impurity) = 0;
    dna = ml_preprocess(dnaimg,[],'ml','yesbgsub');
    dnamask = imopen(dna>0,strel('disk',5));
    dnamask = imfill(dnamask,'holes');
end
dnamask = bwareaopen(dnamask,800);

dnaimg(~dnamask) = 0;
if nnz(dnaimg)==0
    objfeat = [];
    objfluo = [];
    num_dna = 0;
    return
end

% Calculate protein object morphological features
dnaobjs = ml_findobjs(dnaimg);
num_dna = length(dnaobjs);
for k = 1:num_dna
    objfeat(k,:) = tz_objfeat...
        (dnaobjs{k},[],struct('featset',{{'mor','skl'}}));
    objfluo(k) = sum(dnaobjs{k}(:,end));
    protcof(k,:) = ml_calcobjcof(dnaobjs{k});
end
objfluo = objfluo';

% Two DNA related features
dnafeat = [zeros(num_dna,1),ones(num_dna,1)];
objfeat(:,2:3) = dnafeat;