function accur = gnf_LOOCVt1test(T1t)
% Run cross validation to test the accuracy of unmixing corresponding to
% each designated accuracy level T1t
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

modelpath = './inter_results/T1/model';
tunepath = './inter_results/T1/tune';

[X,Y] = meshgrid(8,1:7);
wells = [X,Y;Y,X];
groundtruth = [ones(7,1),zeros(7,1);zeros(7,1),ones(7,1)];
groundtruth(:,end+1) = 0;
groundtruthdna = [zeros(14,2),ones(14,1)];

for i = 1:7
    disp(num2str(i))
    load([modelpath '/model',num2str(i),'.mat'])
    load([tunepath '/t1tune_',num2str(i),'.mat'])
    if ~exist('T1','var')
        T1 = fliplr([0,5*10.^(-8:.5:-5),.0001,.00015,5*10.^(-4.5),...
            .0002:.00005:.0005,.002:.002:.1]);
    end
    t1 = interp1(avgaccur,T1,T1t);
    testidx = [i,i+7];
    testwells = wells(testidx,:);
    testtruth = groundtruth(testidx,:);
    testtruth_dna = groundtruthdna(testidx,:);
    for t = 1:size(testwells,1)
        accuracy(testidx(t)) = gnf_t1test(model,2,testwells(t,:),...
            testtruth(t,:),t1,5,0);
        accuracy_dna(testidx(t)) = gnf_t1test(model,2,testwells(t,:),...
            testtruth_dna(t,:),t1,5,1);
    end
end
accur(1,:) = accuracy;
accur(2,:) = accuracy_dna;

accurpath = './inter_results/T1/xaccurs/';
if ~exist(accurpath,'dir')
    mkdir(accurpath)
end
save([accurpath '/xval-accur_',...
    num2str(T1t*100),'.mat'],'accur')
