function [accuracyf,accuracyn] = gnf_LOOCVt2test(T2CL)
% Run cross validation to test the performance of pattern level outlier
% removal method using ER and the outlier pattern
% T2CL - assigned confidence level to get T2 from the distribution learned
% from the training fold
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

modelpath = './inter_results/T2/model';
tunepath = './inter_results/T2/tune';
evalpath = './inter_results/T2/xaccurs';
if ~exist(evalpath,'dir')
    mkdir(evalpath)
end

[X,Y] = meshgrid([1,3],1:7);
wells = [Y(:),X(:)];
erwells = [16*ones(7,1),(1:7)'];
randseq = [6,3,7,5,1,2,4]';
erwells = erwells(randseq,:);
wells = [wells;erwells];

accuracyf = zeros(1,21);
accuracyn = zeros(1,21);

for i = 1:7
    disp(num2str(i))
    load([modelpath '/model',num2str(i),'.mat'])
    load([tunepath '/errCDF',num2str(i),'.mat'])
    T2f = quantile(Ferr,T2CL);
    T2n = quantile(Nerr,T2CL);
    testidx = [i,i+7,i+14];
    testwells = wells(testidx,:);
    for t = 1:size(testwells,1)
        [accuracyf(testidx(t)),accuracyn(testidx(t))] = gnf_t2test(...
            model,testwells(t,:),T2f,T2n,5);
    end
end

save([evalpath '/xval-accur_',num2str(T2CL*100),'.mat'],...
    'accuracyf','accuracyn')
