function gnf_drugefftanalysis(featready)
% Application of unmixing: analyze on drug treated protein data to
% quantitively describe the relocation fraction
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Feature calculation 
if ~featready
    for i = 13:14               % This step can be computed in parallel
        for j = 1:48
            gnf_wellfeats(i,j)
        end
    end
end

%% Training
[Rm,Cm] = meshgrid([13,14],[1,2,25,26]);
[Rn,Cn] = meshgrid([13,14],[23,24,47,48]);
trainwells = [Rm(:),Cm(:);Rn(:),Cn(:)];
classid = [ones(8,1);2*ones(8,1)];

objectlearned = gnf_objectlearn2(trainwells,classid);
objdistr = gnf_objdistr3(objectlearned);
trainresultname = './inter_results/Drug/trainresult.mat';
if ~exist('./inter_results/Drug/','dir')
    mkdir('./inter_results/Drug/')
end
save(trainresultname,'objectlearned','objdistr')

%% Unmixing
load(trainresultname)
oddwells1 = 3:2:21;
oddwells2 = 27:2:45;
evenwells1 = oddwells1 + 1;
evenwells2 = oddwells2 + 1;
for i = 1:10
    col = [oddwells1(i),evenwells1(i),oddwells2(i),evenwells2(i)];
    [R,C] = meshgrid([13,14],col);
    wells{i} = [R(:),C(:)];
end

for i = 1:10
    disp(['Unmixing the ' num2str(i) 'th wellset'])
    alpha{i} = gnf_unmixwells(objectlearned,objdistr,wells{i});
end

save('./inter_results/Drug/unmixingresult.mat','alpha');
