function [objfeat,nonobjfluo] = gnf_drugimfeat(protimg,tempsize)
% Calculate object features of image of protein processed by drug
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% Preprocessing
se = strel('disk',tempsize);
tophatimg = double(imtophat(protimg,se));
protimg = double(protimg);
[procimg,mask,res] = ml_preprocess(tophatimg,[],'mb','nobgsub');
objimg = protimg;   objimg(~mask) = 0;
cytoimg = protimg;  cytoimg(mask) = 0;

% Calculate protein object morphological features
protobjs = ml_findobjs(objimg);

for k = 1:length(protobjs)
    objfeat(k,:) = tz_objfeat...
        (protobjs{k},[],struct('featset',{{'mor','skl'}}));
    objfluo(k) = sum(protobjs{k}(:,end));
end
objfluo = objfluo';
objfeat = [objfluo,objfeat];

nonobjfluo = sum(cytoimg(:));