function gnf_genfigs(block)
% Generate all figures for the paper
%
% Author: T. Peng
% Created: 12-Jun-2008
% Last Update: 14-Jan-2010
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


resultpath = ['./inter_results/block' int2str(block)];
figurepath = './figures';
if ~exist(figurepath,'dir')
    mkdir(figurepath)
end

fz_str = 6;
fz_num = 6;
dpi = 600;

%% Figure 1
% Created by picture editor

%% Figure S1
load([resultpath '/objlearned.mat'])
aic = objectlearned.aic;
figure
figuresize = [8.7,6.5];
plot(2:length(aic),aic(2:end),'k')
xlabel('Number of object types','fontsize',fz_str)
ylabel('AIC','fontsize',fz_str)
set(gca,'fontsize',fz_num)
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figureS1.tif'])

%% Figure 2
load([resultpath '/objectdistri.mat'])
figure
figuresize = [8.7,6.5];
objhist = squeeze(mean(objnumdistr));
[sorted,idx] = sort(objhist(:,2)-objhist(:,1));
bar(objhist(idx,:),'grouped')
xlabel('Object type index','fontsize',fz_str)
ylabel('Number of objects per image','fontsize',fz_str)
set(gca,'xlim',[0 length(idx)+1])
colormap([0,0,0;1,1,1])
set(gca,'fontsize',fz_num)
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figure2.tif'])

%% Figure S2
figuresize = [8.7,4.3];
figure
for ifig=1:2
    subplot(120+ifig)
    normobj=gnf_normalize(squeeze(objfluodistr(:,:,ifig)),2);
    [sortedvals,sortindx]=sort(mean(normobj));
    %plot(normobj')
    startindex=min(find(sortedvals>0.025));
    disp('number of objects with an average of more than 2.5% of fluorescence')
    disp(size(objfluodistr,2)-startindex+1)
    relobj=squeeze(objfluodistr(:,sortindx(startindex:end),ifig));
    xdiv=repmat(max(relobj,[],1),size(relobj,1),1);
    relobj=relobj./xdiv;
    fcorr = gnf_corrcoef(mitoconcen,relobj);
    disp('Average correlation coefficient of fluorescence vs. probe')
    if ifig==1
        plot(mitoconcen,relobj,'k')
%         set(gca,'XTick',sort(mitoconcen),'Xlim',[min(mitoconcen) max(mitoconcen)])
        set(gca,'Xlim',[min(mitoconcen) max(mitoconcen)])
        xlabel('[Mitotracker]','fontsize',fz_str)
        ylabel('Relative fluorescence','fontsize',fz_str)
        set(gca,'fontsize',fz_num)
        disp(['Mitotracker ' num2str(fcorr)])
    else
        plot(lysoconcen,relobj,'k')
%         set(gca,'XTick',sort(lysoconcen),'Xlim',[min(lysoconcen) max(lysoconcen)])
        set(gca,'Xlim',[min(lysoconcen) max(lysoconcen)])
        xlabel('[Lysotracker]','fontsize',fz_str)
        ylabel('Relative fluorescence','fontsize',fz_str)
        set(gca,'fontsize',fz_num)
        disp(['Lysotracker ' num2str(fcorr)])
    end
end
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figureS2.tif'])

%% Figure 3
% note that this figure also uses variables from objectdistri.mat
load([resultpath '/estimation.mat'])
figure
% the mitoconcen and lysoconcen vectors only contain the lowest six
% concentrations using for building the model - need to add in the highest
% concentration since it is considered when doing the unmixing
rel_mitoconcen = [1; mitoconcen/700];
rel_lysoconcen = [1; lysoconcen/300];
rlm = repmat(rel_mitoconcen,1,length(rel_mitoconcen));
expectedalpha(:,:,2) = rlm./(rlm+repmat(rel_lysoconcen',length(rel_lysoconcen),1));
expectedalpha(:,:,1) = 1-expectedalpha(:,:,2);
unmixcorr(1) = gnf_corrcoef(alpha{1},expectedalpha);
unmixcorr(2) = gnf_corrcoef(alpha{2},expectedalpha);
unmixcorr(3) = gnf_corrcoef(alpha{3},expectedalpha);
disp('Estimated vs. expected fundamental patterns fraction correlation coefficients')
disp(['Linear unmixing: ' num2str(unmixcorr(1))])
disp(['Multinomial unmixing: ' num2str(unmixcorr(2))])
disp(['Fluorescence fraction unmixing: ' num2str(unmixcorr(3))])
figuresize = [8.7 8.7];
subplot(2,2,1),imshow(gnf_heatschema(expectedalpha,32));
title('Predicted pattern fractions','fontsize',fz_str)
subplot(2,2,2),imshow(gnf_heatschema(alpha{1},32));
title('Linear unmixing','fontsize',fz_str)
subplot(2,2,3),imshow(gnf_heatschema(alpha{2},32));
title('Multinomial unmixing','fontsize',fz_str)
subplot(2,2,4),imshow(gnf_heatschema(alpha{3},32));
title('Fluorescence fraction unmixing','fontsize',fz_str)
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figure3_raw.tif'])
gnf_addfig3axis(block)

%% Figure S3
figure
figuresize = [8.7 4.3];
gnf_verify(block)
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figureS3.tif'])

%% Figure 4
figure
figuresize = [8.7 4.3];
gnf_olrmeval
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figure4.tif'])

%% Figure 5
figure
figuresize = [8.7,6.5];
gnf_drugtreatplot
axis tight
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figure5.tif'])
gnf_resulteval

%% Figure S4
figure
figuresize = [8.7,6.5];
load('./inter_results/block2/objrmalys.mat')
for i = 1:length(alpha)
    alphatest = alpha{i}(1:7,1:7,:);
    accurrmobj(i) = gnf_corrcoef(alphatest,expectedalpha);
    accurtrain(i) = (mean(squeeze(alpha{i}(1:7,8,2))) ...
        + mean(squeeze(alpha{i}(8,1:7,1))))/2;
end
plot(0:length(alpha)-1,accurrmobj,'k^','MarkerFaceColor','k','MarkerSize',2.5)
hold on
plot(0:length(alpha)-1,accurrmobj,'k')

plot(0:length(alpha)-1,accurtrain,'k^','MarkerFaceColor','k','MarkerSize',2.5)
hold on
plot(0:length(alpha)-1,accurtrain,'k')

xlabel('Number of object types removed','fontsize',fz_str)
ylabel('Accuracy/CorrCoeff of unmixing','fontsize',fz_str)
set(gca,'fontsize',fz_num)
set(gcf,'PaperUnits','centimeters','PaperPosition',[0,0,figuresize])
print('-dtiff',['-r' int2str(dpi)],[figurepath '/figureS4.tif'])