function map = gnf_heatschema(alpha,w)
% FUNCTION GNF_HEATSCHEMA draw a grayscale heat-map by the fraction of
% protein patterns in different wells
%
% Author: T. Peng
% Created: 12-Apr-2008
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

block = zeros(w);
block(2:end-1,2:end-1) = 1;
cross = ~diag(true(w,1)) & ~fliplr(diag(true(w,1)));
emptyblock = double(logical(block)&cross);

map = [];
for i = 1:size(alpha,1)
    row = [];
    for j = 1:size(alpha,2)
        if ~isnan(alpha(i,j,1)) && ~isnan(alpha(i,j,2))
            row = [row block*255*alpha(i,j,2)];
        else
            row = [row emptyblock*255];
        end
    end
    map = [map;row];
end

map = uint8(map);