function model = gnf_modeltraining2(noisethresh,path,...
    wells,patternid)
% Train an unmixing model for pattern level outlier removal
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Train with selected wells
nwells = size(wells,1);
allfeat = [];
allfluo = [];
classid = [];
imid = [];
imcount = 0;
for i = 1:nwells
    for k = 1:35
        name = sprintf('%s/%03d%03d/feat%d.mat',path,...
            wells(i,1),wells(i,2),k);
        if exist(name,'file')
            load(name)        
            if ~isempty(objfluo)
                allfeat = [allfeat;objfeat];
                allfluo = [allfluo;objfluo];
                imcount = imcount + 1;
                imid = [imid;imcount*ones(size(objfeat,1),1)];
                classid = [classid;patternid(i)*ones(size(objfeat,1),1)];
            end
        end
        if i < nwells && patternid(i+1) ~= patternid(i)
            imcount = 0;
        end
    end
end
allfeat(allfluo<=noisethresh,:) = [];
classid(allfluo<=noisethresh) = [];
imid(allfluo<=noisethresh) = [];
allfluo(allfluo<=noisethresh) = [];

%% Clustering
[zscoredfeat,offset,scale] = ml_zscore(allfeat);
MaxCluster = 20;
objtypeall = cell(MaxCluster,1);
pdfall = cell(MaxCluster,1);
objpostall = cell(MaxCluster,1);
tmpdir = gnf_tempname;
for ncluster = 2:MaxCluster
    disp(['Doing ' num2str(ncluster) ' clustering']);
    [objpostall{ncluster},pdfall{ncluster}] = gnf_objcluster...
        (zscoredfeat,imid,classid,'kmeans',ncluster,'nosave',...
        ['./inter_results/T2/tmp' tmpdir '/train']);
    [objtypeall{ncluster},non] = find(objpostall{ncluster}');
    [aic(ncluster),bic(ncluster)] = tz_aicbic(zscoredfeat,objtypeall{ncluster},'euc');
end
[aic_min,best_ncluster] = min(aic(2:end));
best_ncluster = best_ncluster + 1;
objtype = objtypeall{best_ncluster};
objpost = objpostall{best_ncluster};
pdf = pdfall{best_ncluster};

model = struct(...
    'zscoredfeat',zscoredfeat,...
    'allfeat',allfeat,...
    'allfluo',allfluo,...
    'imid',imid,...
    'classid',classid,...
    'aic',aic,...
    'bic',bic,...
    'offset',offset,...
    'scale',scale,...
    'objtype',objtype,...
    'pdf',pdf,...
    'objpost',objpost,...
    'best_ncluster',best_ncluster);

rmdir(['./inter_results/T2/tmp' tmpdir '/train'],'s')