function objdistr = gnf_objdistr2(objlearned)
%GNF_OBJDISTR calculates distribution of objects and object fluorescence
%fraction of fundamental patterns
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Initilization
fluo = objlearned.allfluo;
imid = objlearned.imid;
classid = objlearned.classid;
objpost = objlearned.objpost;

%% Calculate object frequency and fluorescence distribution
nclusters = size(objpost,2);
nclasses = max(classid);
for c = 1:nclasses
    nims = length(unique(imid(classid==c)));
    objnum(c,:) = sum(objpost(classid==c,:));
    objnumdistr(c,:) = objnum(c,:)/nims;
    fluopost = objpost.*repmat(fluo,[1,nclusters]);
    objfluo(c,:) = sum(fluopost(classid==c,:));
    objfluodistr(c,:) = objfluo(c,:)/nims;
end

%% Calculate object frequency and fluorescence fraction distribution
objnumfrac = gnf_normalize(objnumdistr,2);
objfluofrac = gnf_normalize(objfluodistr,2);

%% Output resutls
objdistr.objnumdistr = objnumdistr;
objdistr.objfluodistr = objfluodistr;
objdistr.objnumfrac = objnumfrac;
objdistr.objfluofrac = objfluofrac;