function objectlearned = gnf_objectlearn(noisethresh,path,block)
% Learn unmixing model for mito-lyso data analysis
%
% Author: T. Peng
% Created: 12-Apr-2008
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

blockdir = ['./inter_results/block' int2str(block)];
if ~exist(blockdir,'dir')
    mkdir(blockdir)
end
%% Train with pure dye tagged wells

mitofeat = [];
mitofluo = [];
mitoimid = [];
mitoojid = [];
for t = 1:7
    for k = 1:35
        name = [path '/well' num2str(block) '8' num2str(t) '/feat'...
            num2str(k) '.mat'];
        if exist(name,'file')
            load(name)        
            mitofeat = [mitofeat;objfeat];
            mitofluo = [mitofluo;objfluo];
            mitoimid = [mitoimid;(100*t+k)*ones(size(objfeat,1),1)];
            mitoojid = [mitoojid;[1:size(objfeat,1)]'];
        end
    end
end
mitofluo(mitofeat(:,1)<=noisethresh,:) = [];
mitoimid(mitofeat(:,1)<=noisethresh,:) = [];
mitoojid(mitofeat(:,1)<=noisethresh,:) = [];
mitofeat(mitofeat(:,1)<=noisethresh,:) = [];
mitoid = ones(size(mitofeat,1),1);

lysofeat = [];
lysofluo = [];
lysoimid = [];
lysoojid = [];
for t = 1:7
    for k = 1:35
        name = [path '/well' num2str(block) num2str(t) '8/feat'...
            num2str(k) '.mat'];
        if exist(name,'file')
            load(name);
            lysofeat = [lysofeat;objfeat];
            lysofluo = [lysofluo;objfluo];
            lysoimid = [lysoimid;(100*t+k)*ones(size(objfeat,1),1)];
            lysoojid = [lysoojid;(1:size(objfeat,1))'];
        end
    end
end
lysofluo(lysofeat(:,1)<=noisethresh,:) = [];
lysoimid(lysofeat(:,1)<=noisethresh,:) = [];
lysoojid(lysofeat(:,1)<=noisethresh,:) = [];
lysofeat(lysofeat(:,1)<=noisethresh,:) = [];
lysoid = 2*ones(size(lysofeat,1),1);

allfeat = [mitofeat;lysofeat];
allfluo = [mitofluo;lysofluo];
imid = [mitoimid;lysoimid];
ojid = [mitoojid;lysoojid];
classid = [mitoid;lysoid];

save(['./inter_results/block' int2str(block) '/unmixtrain.mat'],...
    'allfeat','allfluo','imid','ojid','classid');

%% Clustering
[zscoredfeat,offset,scale] = ml_zscore(allfeat);
MaxK = 35;
objtypeall = cell(MaxK,1);
pdfall = cell(MaxK,1);
objpostall = cell(MaxK,1);
tmpdir = gnf_tempname;
for ncluster = 2:MaxK
    disp(['Doing ' num2str(ncluster) ' clustering']);
    [objpostall{ncluster},pdfall{ncluster}] = gnf_objcluster...
        (zscoredfeat,imid,classid,'kmeans',ncluster,'nosave',...
        ['./inter_results/block' num2str(block) '/tmp' tmpdir '/train']);
    [objtypeall{ncluster},non] = find(objpostall{ncluster}');
    [aic(ncluster),bic(ncluster)] = ...
        tz_aicbic(zscoredfeat,objtypeall{ncluster},'euc');
end
[aic_min,best_ncluster] = min(aic(2:end));
best_ncluster = best_ncluster + 1;
objtype = objtypeall{best_ncluster};
objpost = objpostall{best_ncluster};
pdf = pdfall{best_ncluster};

objectlearned = struct(...
    'zscoredfeat',zscoredfeat,...
    'allfeat',allfeat,...
    'allfluo',allfluo,...
    'imid',imid,...
    'classid',classid,...
    'aic',aic,...
    'bic',bic,...
    'offset',offset,...
    'scale',scale,...
    'objtype',objtype,...
    'pdf',pdf,...
    'objpost',objpost,...
    'best_ncluster',best_ncluster);

save(['./inter_results/block' int2str(block)...
    '/objlearned.mat'],'objectlearned');

rmdir(['./inter_results/block' num2str(block) '/tmp' tmpdir],'s')