function objectlearned = gnf_objectlearn2(wells,class)
% Learn an unmixing model for drug data
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Train with selected wells
nwells = size(wells,1);
allfeat = [];
classid = [];
cellid = [];
cellcount = 0;
for i = 1:nwells
    [feats,cytofluo,idx] = gnf_loadwellfeats(wells(i,:));
    allfeat = [allfeat;feats];
    cellid = [cellid;idx+cellcount];
    cellcount = max(cellid);
    wellcytofluo(i) = sum(cytofluo);
    classid = [classid;class(i)*ones(size(idx))];
end
allfluo = allfeat(:,1);
allfeat(:,[1 3 4]) = [];
for i = 1:max(class)
    Nobjfluo(i) = sum(wellcytofluo(class==i));
end

%% Clustering
[zscoredfeat,offset,scale] = ml_zscore(allfeat);
MaxCluster = 15;
objtypeall = cell(MaxCluster,1);
pdfall = cell(MaxCluster,1);
objpostall = cell(MaxCluster,1);
for ncluster = 2:MaxCluster
    disp(['Doing ' num2str(ncluster) ' clustering']);
    [objpostall{ncluster},pdfall{ncluster}] = gnf_objcluster...
        (zscoredfeat,cellid,classid,'kmeans',...
        ncluster,'nosave','./inter_results/Drug/tmp');
    [objtypeall{ncluster},non] = find(objpostall{ncluster}');
    [aic(ncluster),bic(ncluster)] = tz_aicbic(zscoredfeat,objtypeall{ncluster},'euc');
end
[aic_min,best_ncluster] = min(aic(2:end));
best_ncluster = best_ncluster + 1;
objtype = objtypeall{best_ncluster};
objpost = objpostall{best_ncluster};
pdf = pdfall{best_ncluster};

objectlearned = struct(...
    'allfeat',allfeat,...
    'zscoredfeat',zscoredfeat,...
    'allfluo',allfluo,...
    'Nobjfluo',Nobjfluo,...
    'cellid',cellid,...
    'classid',classid,...
    'aic',aic,...
    'bic',bic,...
    'offset',offset,...
    'scale',scale,...
    'objtype',objtype,...
    'pdf',pdf,...
    'objpost',objpost,...
    'best_ncluster',best_ncluster);

rmdir('./inter_results/Drug/tmp','s')