function gnf_objoutlieranalysis(featready)
% Performs both object outlier analysis and pattern outlier analysis to
% provide supporting data for figure 7a
%
% Author: T. Peng
% Created: 12-Sep-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if ~exist('./inter_results/T1','dir')
    mkdir('./inter_results/T1');
end

%% Object outlier analysis
% Object feature calculation
if ~featready
    blockIndex = 2;
    rowIndex = 8;
    colIndex = 1:7;
    gnf_allfeats(rowIndex,colIndex,blockIndex);
    gnf_allDNAfeats(rowIndex,colIndex,blockIndex);
    rowIndex = 1:7;
    colIndex = 8;
    gnf_allfeats(rowIndex,colIndex,blockIndex);
    gnf_allDNAfeats(rowIndex,colIndex,blockIndex);
end

% Parameter T1 tuning: train sub-LOOCV models
for N = 1:7             % This step can be computed in parallel
    for M = N+1:7
        gnf_LOOCVtrain_sub(N,M);
    end
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(1,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(2,3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(2,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(2,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(2,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(2,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(3,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(3,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(3,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(3,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(4,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(4,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(4,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(5,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(5,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub(6,7)"

% Parameter T1 tuning: generate confidence-leve vs. accuracy level curve
T1 = fliplr([0,5*10.^(-8:.5:-5),.0001,.00015,5*10.^(-4.5),...
    .0002:.00005:.0005,.002:.002:.1]);
for N = 1:7                 % This step can be computed in parallel
    gnf_LOOCVt1test_sub(N,T1);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(1)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test_sub(7)"

% T1 performance test: LOOCV on protein and dna image sets
% Train LOOCV models
for N = 1:7             % This step can be computed in parallel
    gnf_LOOCVtrain(N);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(1)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain(7)"

% Accuracy of outlier exclusion on subjective accuracy level
for T1t = 0.80:0.01:0.95            % This step can be computed in parallel
    gnf_LOOCVt1test(T1t);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.95)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.94)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.93)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.92)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.91)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.90)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.89)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.88)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.87)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.86)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.85)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.84)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.83)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.82)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.81)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt1test(.80)"
