function gnf_olrmeval
% Plot the outlier removal analysis curves
%
% Author: T. Peng
% Created: 10-Sep-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

t1path = './inter_results/T1/xaccurs';
t2path = './inter_results/T2/xaccurs';

fz_str = 6;
fz_num = 6;

%% Plot both inlier and outlier accuracy vs. designated accuracy level
T1t = [80:2:95,95];
inavgaccur = zeros(1,length(T1t));
outavgaccur = zeros(1,length(T1t));
for i = 1:length(T1t)
    matname = sprintf('%s/xval-accur_%d.mat',t1path,T1t(i));
    load(matname)
    inlier = accur(1,:);
    outlier = accur(2,:);
    inlier(isnan(inlier)) = [];
    outlier(isnan(outlier)) = [];
    inavgaccur(i) = mean(inlier);
    outavgaccur(i) = 1 - mean(outlier);
end

subplot(1,2,1)
plot(T1t/100,inavgaccur,'k')
hold on
plot(T1t/100,outavgaccur,'k')
plot(T1t/100,inavgaccur,'ks','MarkerSize',2.5,'MarkerFaceColor','k')
plot(T1t/100,outavgaccur,'k^','MarkerSize',2.5,'MarkerFaceColor','k')
axis([.80,.95,0,1])
xlabel('Inlier unmixing accuracy level','fontsize',fz_str)
ylabel('Inlier fluorescence fraction','fontsize',fz_str)
set(gca,'fontsize',fz_num)


%% Plot both inlier and outlier accuracy vs. designated confidence level
T2cl = [50:5:75,80:2:100];
inavgaccur = zeros(1,length(T2cl));
outavgaccur = zeros(1,length(T2cl));
for i = 1:length(T2cl)
    matname = sprintf('%s/xval-accur_%d.mat',t2path,T2cl(i));
    load(matname)
    inlier = accuracyf(1:14);
    outlier = accuracyf(15:21);
    inlier(isnan(inlier)) = [];
    outlier(isnan(outlier)) = [];
    inavgaccur(i) = mean(inlier);
    outavgaccur(i) = mean(outlier);
end

subplot(1,2,2)
plot(T2cl/100,inavgaccur,'k')
hold on
plot(T2cl/100,outavgaccur,'k')
plot(T2cl/100,inavgaccur,'ks','MarkerSize',2.5,'MarkerFaceColor','k')
plot(T2cl/100,outavgaccur,'k^','MarkerSize',2.5,'MarkerFaceColor','k')
xlabel('Inlier confidence level','fontsize',fz_str)
ylabel('Inlier image fraction','fontsize',fz_str)
axis([0.5 1 0 1])
set(gca,'XTick',[0.5,0.6,0.7,0.8,0.9,1])
set(gca,'fontsize',fz_num)