function gnf_patternoutlieranalysis(featready)
% Performs pattern outlier analysis to provide supporting data for figure
% 7b
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if ~exist('./inter_results/T2','dir')
    mkdir('./inter_results/T2');
end

%% Object outlier analysis
% Object feature calculation
[X,Y] = meshgrid([1,3],1:7);
wells = [Y(:),X(:)];
erwells = [16*ones(7,1),(1:7)'];
wells = [wells;erwells];
if ~featready             % This step can be computed in parallel
    for k = 1:size(wells,1)
        gnf_MLEfeats(wells(k,1),wells(k,2))
    end
end

%% Parameter T2 tuning: train sub-LOOCV models
for N = 1:7             % This step can be computed in parallel
    for M = N+1:7
        gnf_LOOCVtrain_sub2(N,M);
    end
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(1,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(2,3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(2,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(2,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(2,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(2,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(3,4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(3,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(3,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(3,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(4,5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(4,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(4,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(5,6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(5,7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain_sub2(6,7)"

%% Parameter T2 tuning: learn fitting error distribution
for N = 1:7             % This step can be computed in parallel
    gnf_LOOCVerrortest(N);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(1)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVerrortest(7)"

%% T2 performance test: LOOCV on mito/lyso and ER image sets
% Train LOOCV models
for N = 1:7             % This step can be computed in parallel
    gnf_LOOCVtrain2(N);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(1)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(2)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(3)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(4)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVtrain2(7)"

% Accuracy of outlier exclusion on subjective confidence level
for t2cl = [.50:.05:.75,.80:.02:1] % This step can be computed in parallel
    gnf_LOOCVt2test(t2cl);
end
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.5)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.55)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.6)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.65)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.7)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.75)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.80)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.82)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.84)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.86)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.88)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.90)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.92)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.94)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.96)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(.98)"
% qarray -m "addpath(genpath('./functions'));gnf_LOOCVt2test(1)"