function [alpha] = gnf_rmobjtypes(objlearned,noisethresh,path,block,wells)
% Do analysis by removing non-discriminative objects
%
% Author: T. Peng
% Created: 10-Sep-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Initialization
zscoredfeat = objlearned.zscoredfeat;
offset = objlearned.offset;
scale = objlearned.scale;
objtype = objlearned.objtype;
ncluster = objlearned.best_ncluster;

%% Object distribution learning
objdistr = gnf_objdistr2(objlearned);
objnumdistr = objdistr.objnumdistr;
[rmorder,distpw] = gnf_rmsimobj(objnumdistr);

%% Unmixing
for w = 1:size(wells,1)
    x = wells(w,1);
    y = wells(w,2);
    % Load image features for each well
        fname = [path '/well' num2str(100*block+10*x+y)];
        disp(['unmixing well ' num2str(100*block+10*x+y)]);
        if exist(fname,'dir')
            mixfeat = [];
            mixfluo = [];
            miximid = [];
            for k = 1:35
                featname = [fname '/feat' num2str(k) '.mat'];
                if exist(featname,'file')
                    load(featname);
                    mixfeat = [mixfeat;objfeat];
                    mixfluo = [mixfluo;objfluo];
                    miximid = [miximid;k*ones(size(objfeat,1),1)];
                end
            end
            % Exclude noise objects
            mixfluo(mixfeat(:,1)<=noisethresh,:) = [];
            miximid(mixfeat(:,1)<=noisethresh,:) = [];
            mixfeat(mixfeat(:,1)<=noisethresh,:) = [];
            zsoremixfeat = ml_zscore(mixfeat,offset,scale);
            % Recoginize object types
            testobjtype = tz_classify(zsoremixfeat,zscoredfeat,objtype,'dist',1);
            testobjdistr = hist(testobjtype,ncluster);
            imnum = length(unique(miximid));
            for T = 0:ncluster-2
                % Approach 1: Object number frequency linear unmixing
                r = nnz(distpw<T/10);
                A = objnumdistr;
                b = testobjdistr/imnum;
                if r==0
                    alpha{T+1}(w,:) = gnf_solvemix(A',b','normalize')';
                elseif r>0 && r<=(ncluster-2)
                    b(rmorder(1:r)) = [];
                    A(:,rmorder(1:r)) = [];
                    alpha{T+1}(w,:) = gnf_solvemix(A',b','normalize')';
                end
            end
        end
end
