function [alpha] = gnf_rmsimobjalys(noisethresh,path,block)
% Do analysis by removing non-discriminative objects
%
% Author: T. Peng
% Created: 10-Sep-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Initialization
load(['./inter_results/block' int2str(block) '/unmixtrain.mat'])
load(['./inter_results/block' int2str(block) '/objlearned.mat'])
objlearned = objectlearned;

zscoredfeat = objlearned.zscoredfeat;
allfeat = objlearned.allfeat;
allfluo = objlearned.allfluo;
imid = objlearned.imid;
classid = objlearned.classid;
offset = objlearned.offset;
scale = objlearned.scale;
objtype = objlearned.objtype;
pdf = objlearned.pdf;
objpost = objlearned.objpost;
ncluster = objlearned.best_ncluster;

%% Object distribution learning
[objnumdistr,objfluodistr,fluo_concen_k,avgfluofrac] = ...
    gnf_objdistr(objpost,allfluo,imid,classid,block);
A_fluo = repmat(fluo_concen_k',[ncluster,1]).*avgfluofrac;
Asum = sum(A_fluo);
A = [mean(objnumdistr(:,:,1));mean(objnumdistr(:,:,2))];

rmorder = gnf_rmsimobj(objnumdistr);
rmorder2 = gnf_rmsimobj(objfluodistr);

%% Unmixing
alpha = cell(ncluster-2,1);
for x = 1:8
    for y = 1:8
        % Load image features for each well
        fname = [path '/well' num2str(100*block+10*x+y)];
        disp(['unmixing well ' num2str(100*block+10*x+y)]);
        if exist(fname,'dir')
            mixfeat = [];
            mixfluo = [];
            miximid = [];
            for k = 1:35
                featname = [fname '/feat' num2str(k) '.mat'];
                if exist(featname,'file')
                    load(featname);
                    mixfeat = [mixfeat;objfeat];
                    mixfluo = [mixfluo;objfluo];
                    miximid = [miximid;k*ones(size(objfeat,1),1)];
                end
            end
            % Exclude noise objects
            mixfluo(mixfeat(:,1)<=noisethresh,:) = [];
            miximid(mixfeat(:,1)<=noisethresh,:) = [];
            mixfeat(mixfeat(:,1)<=noisethresh,:) = [];
            zsoremixfeat = ml_zscore(mixfeat,offset,scale);
            % Recoginize object types
            testobjtype = tz_classify(zsoremixfeat,zscoredfeat,objtype,'dist',1);
            testobjdistr = hist(testobjtype,ncluster);
            imnum = length(unique(miximid));
            for k = 1:ncluster
                totalfluo(k) = sum(mixfluo(testobjtype==k))/imnum;
            end
            for r = 0:ncluster-2
                % Approach: Object number frequency linear unmixing
                b = testobjdistr/imnum;
                Atmp = A;
                if r>0
                    b(rmorder(1:r)) = [];
                    Atmp = A;
                    Atmp(:,rmorder(1:r)) = [];
                end
                alpha{r+1}(x,y,:) = gnf_solvemix(Atmp',b','normalize');
            end
        end
    end
end

save(['./inter_results/block' int2str(block) '/objrmalys.mat'],...
        'alpha');