function accuracy = gnf_t1test(model,block,wells,groundtruth,T1,...
    noisethresh,isdnawell)
% Run cross validation to find best outlier detection threshold T1
% ACCURACY - accuracy of unmixing
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% Load all testing features include DNA object feature
featpath = './inter_results/features/MitoLyso/prot';
dnafeatpath = './inter_results/features/MitoLyso/DNA';

nwells = size(wells,1);
mixfeat = [];
mixfluo = [];
imid = [];
imcount = 0;
for i = 1:nwells
    for k = 1:35
        if ~isdnawell
            name = sprintf('%s/well%d%d%d/feat%d.mat',featpath,block,...
                wells(i,1),wells(i,2),k);
            if exist(name,'file')
                load(name)        
                if ~isempty(objfeat)
                    mixfeat = [mixfeat;objfeat];
                    mixfluo = [mixfluo;objfluo];
                    imcount = imcount + 1;
                    imid = [imid;imcount*ones(size(objfeat,1),1)];
                end
            end
        else
            dnafeatname = sprintf('%s/%d%d%d/feat%d.mat',...
                dnafeatpath,block,wells(i,1),wells(i,2),k);
            if exist(dnafeatname,'file')
                load(dnafeatname)
                if ~isempty(objfeat)
                    mixfeat = [mixfeat;objfeat];
                    mixfluo = [mixfluo;objfluo];
                    imid = [imid;imcount*ones(size(objfeat,1),1)];
                end
            end
        end
    end
end
mixfeat(mixfluo<=noisethresh,:) = [];
imid(mixfluo<=noisethresh) = [];
mixfluo(mixfluo<=noisethresh) = [];

% Initialize model
zscoredfeat = model.zscoredfeat;
offset = model.offset;
scale = model.scale;
objtype = model.objtype;
pdf = model.pdf;
ncluster = model.best_ncluster;

g = logical(groundtruth);
if isempty(mixfeat)
    result = [0,0,1];
    accuracy = result(g);
    return
end

% Object outlier detection
testfeat = ml_zscore(mixfeat,offset,scale);
testobjtype = tz_classify(testfeat,zscoredfeat,objtype,'dist',1);
alpha = T1;
outlieridx = [];
for k = 1:ncluster
    clusterfeat = zscoredfeat(objtype==k,:);
    mu = mean(clusterfeat);
    C = cov(clusterfeat);
    isoutlier = tp_isoutlier(testfeat(testobjtype==k,:),mu,C,alpha);
    typekobjidx = find(testobjtype==k);
    outlieridx = [outlieridx;typekobjidx(isoutlier)];
end
% Remove outlier
outlierfluo = mixfluo(outlieridx);
outlierfrac = sum(outlierfluo)/sum(mixfluo);
testobjtype(outlieridx) = [];
mixfluo(outlieridx) = [];
imid(outlieridx) = [];

% Object number and fluorescence distribution
objdistr = gnf_objdistr2(model);
objnumfrac = objdistr.objnumfrac;
objfluofrac = objdistr.objfluofrac;

% Unmixing test
testobjdistr = hist(testobjtype,ncluster);
imnum = length(unique(imid));
for k = 1:ncluster
    totalfluo(k) = sum(mixfluo(testobjtype==k))/imnum;
end
% Approach 3: Fluorescence fraction concentration estimation and unmixing
mixfluofrac = gnf_normalize(totalfluo,2);
result = gnf_solvemix(objfluofrac',mixfluofrac','normalize');

% Output results
x = result;
x = x * (1-outlierfrac);
x(3) = outlierfrac;
accuracy = x(g);