function [Funmixerr,Nunmixerr] = gnf_unmixerrortest(model,wells,noisethresh)
% Retrieve the distribution of unmixing errors on a population of images in
% a single well or several wells
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

featpath = './inter_results/features/MitoLysoER';

nwells = size(wells,1);
mixfeat = [];
mixfluo = [];
imid = [];
imcount = 0;
for i = 1:nwells
    for k = 1:35
        name = sprintf('%s/%03d%03d/feat%d.mat',featpath,...
            wells(i,1),wells(i,2),k);
        if exist(name,'file')
            load(name)
            if ~isempty(objfeat)
                mixfeat = [mixfeat;objfeat];
                mixfluo = [mixfluo;objfluo];
                imcount = imcount + 1;
                imid = [imid;imcount*ones(size(objfeat,1),1)];
            end
        end
    end
end
mixfeat(mixfluo<=noisethresh,:) = [];
imid(mixfluo<=noisethresh) = [];
mixfluo(mixfluo<=noisethresh) = [];

% Initialize model
zscoredfeat = model.zscoredfeat;

offset = model.offset;
scale = model.scale;
objtype = model.objtype;

ncluster = model.best_ncluster;

testfeat = ml_zscore(mixfeat,offset,scale);
testobjtype = tz_classify(testfeat,zscoredfeat,objtype,'dist',1);

% Object number and fluorescence distribution
objdistr = gnf_objdistr2(model);
objnumfrac = objdistr.objnumfrac;
objfluofrac = objdistr.objfluofrac;

% Unmixing test
allobjtype = testobjtype;
allfluo = mixfluo;
Nunmixerr = zeros(imcount,ncluster);
Funmixerr = zeros(imcount,ncluster);
for i = 1:imcount
    testobjtype = allobjtype(imid==i);
    mixfluo = allfluo(imid==i);
    testobjdistr = hist(testobjtype,ncluster);
    totalfluo = zeros(1,ncluster);
    for k = 1:ncluster
        totalfluo(k) = sum(mixfluo(testobjtype==k));
    end
    % Approach 1: Object number frequency linear unmixing
    b = gnf_normalize(testobjdistr,2);
    A = objnumfrac;
    x = gnf_solvemix(A',b','normalize');
    err = b'-A'*x;
    Nunmixerr(i,:) = err';
    % Approach 3: Fluorescence fraction concentration estimation and unmixing
    mixfluofrac = gnf_normalize(totalfluo,2);
    x = gnf_solvemix(objfluofrac',mixfluofrac','normalize');

    % Calculate unmxing fitting error
    A = objfluofrac';
    o = mixfluofrac';
    err = o-A*x;
    Funmixerr(i,:) = err';
end
