function [alpha,tempfluo] = gnf_unmixwells(objlearned,objdistr,wells)
% Unmixing a single well of the drug date after model is trained
%
% Author: T. Peng
% Created: 15-Apr-2009
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Initialization
zscoredfeat = objlearned.zscoredfeat;
% allfluo = objlearned.allfluo;
% Nobjfluo = objlearned.Nobjfluo;
% cellid = objlearned.cellid;
% classid = objlearned.classid;
offset = objlearned.offset;
scale = objlearned.scale;
objtype = objlearned.objtype;
pdf = objlearned.pdf;
% objpost = objlearned.objpost;
ncluster = objlearned.best_ncluster;

objnumdistr = objdistr.objnumdistr;
objfluodistr = objdistr.objfluodistr;
% objnumfrac = objdistr.objnumfrac;
objfluofrac = objdistr.objfluofrac;

%% Unmixing

nwells = size(wells,1);
mixfeat = [];
mixcellid = [];
cellcount = 0;
% Load image features from each well to be unmixed
for i = 1:nwells
    [feats,cytofluo,idx] = gnf_loadwellfeats(wells(i,:));
    mixfeat = [mixfeat;feats];
    mixcellid = [mixcellid;idx+cellcount];
    cellcount = max(mixcellid);
    wellcytofluo(i) = sum(cytofluo);
end
mixfluo = mixfeat(:,1);
mixfeat(:,1) = [];
mixfeat(:,2:3) = [];
mixfeat = ml_zscore(mixfeat,offset,scale);
mixNobjfluo = sum(wellcytofluo);

alpha = cell(4,1);
% Recoginize object types
testobjtype = tz_classify(mixfeat,zscoredfeat,objtype,'dist',1);
testobjdistr = hist(testobjtype,ncluster);
cellnum = length(unique(mixcellid));
for k = 1:ncluster
    totalfluo(k) = sum(mixfluo(testobjtype==k))/cellnum;
end
mixNobjfluo = mixNobjfluo/cellnum;
% Approach 1: Object number frequency linear unmixing
b = testobjdistr/cellnum;
A = objnumdistr;
A = gnf_normalize(A,2);
b = gnf_normalize(b,2);
alpha{1} = gnf_lsqlin(A',b');
% alpha{1} = gnf_solvemix(A',b','normalize');
% Approach 2: Object number fraction multinomial unmixing
alpha{2}(1,1) = tz_fitmnom2(b,pdf(1,:),pdf(2,:));
alpha{2}(2,1) = 1 - alpha{2}(1,1);
% Approach 3: Fluorescence fraction unmixing
objfluofracwrtNobj = gnf_normalize(objfluodistr(:,2:end),2);
mixfluofrac = gnf_normalize(totalfluo,2);
alpha{3} = gnf_solvemix(objfluofracwrtNobj',mixfluofrac','normalize');
% Approach 4: Fluorescence fraction unmixing with non-object type
mixfluofrac = gnf_normalize([mixNobjfluo,totalfluo],2);
alpha{4} = gnf_solvemix(objfluofrac',mixfluofrac','normalize');
tempfluo = [mixNobjfluo,totalfluo];