function [alpha,concentration] = ...
    gnf_unmixwolrm(objlearned,noisethresh,path,block,T1,ysave)
% Unmixing forcing outlier removal, even no outlier patterns in mixture
%
% Author: T. Peng
% Created: 08-Jan-2010
% Last Update: 10-Jan-2010
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%% Initialization
if isempty(objlearned)
    load(['./inter_results/block' int2str(block) '/unmixtrain.mat'])
    load(['./inter_results/block' int2str(block) '/objlearned.mat'])
    objlearned = objectlearned;
end
    zscoredfeat = objlearned.zscoredfeat;
    allfeat = objlearned.allfeat;
    allfluo = objlearned.allfluo;
    imid = objlearned.imid;
    classid = objlearned.classid;
    offset = objlearned.offset;
    scale = objlearned.scale;
    objtype = objlearned.objtype;
    pdf = objlearned.pdf;
    objpost = objlearned.objpost;
    ncluster = objlearned.best_ncluster;

%% Object distribution learning
[objnumdistr,objfluodistr,fluo_concen_k,avgfluofrac] = ...
    gnf_objdistr(objpost,allfluo,imid,classid,block);
A_fluo = repmat(fluo_concen_k',[ncluster,1]).*avgfluofrac;
Asum = sum(A_fluo);

%% Unmixing
alpha = cell(3,1);
for x = 1:7
    for y = 1:7
        % Load image features for each well
        fname = [path '/well' num2str(100*block+10*x+y)];
        disp(['unmixing well ' num2str(100*block+10*x+y)]);
        if exist(fname,'dir')
            mixfeat = [];
            mixfluo = [];
            miximid = [];
            for k = 1:35
                featname = [fname '/feat' num2str(k) '.mat'];
                if exist(featname,'file')
                    load(featname);
                    mixfeat = [mixfeat;objfeat];
                    mixfluo = [mixfluo;objfluo];
                    miximid = [miximid;k*ones(size(objfeat,1),1)];
                end
            end
            % Exclude noise objects
            mixfluo(mixfeat(:,1)<=noisethresh,:) = [];
            miximid(mixfeat(:,1)<=noisethresh,:) = [];
            mixfeat(mixfeat(:,1)<=noisethresh,:) = [];
            zsoremixfeat = ml_zscore(mixfeat,offset,scale);
            % Recoginize object types
            testobjtype = tz_classify(zsoremixfeat,zscoredfeat,objtype,'dist',1);
            
            outlieridx = [];
            for k = 1:ncluster
                clusterfeat = zscoredfeat(objtype==k,:);
                mu = mean(clusterfeat);
                C = cov(clusterfeat);
                isoutlier = tp_isoutlier(zsoremixfeat(testobjtype==k,:),mu,C,T1);
                typekobjidx = find(testobjtype==k);
                outlieridx = [outlieridx;typekobjidx(isoutlier)];
            end
            % Remove outlier
            outlierfluo = mixfluo(outlieridx);
            testobjtype(outlieridx) = [];
            mixfluo(outlieridx) = [];
            miximid(outlieridx) = [];
            
            
            testobjdistr = hist(testobjtype,ncluster);
            imnum = length(unique(miximid));
            for k = 1:ncluster
                totalfluo(k) = sum(mixfluo(testobjtype==k))/imnum;
            end
            % Approach 1: Object number frequency linear unmixing
            b = testobjdistr/imnum;
            A = [mean(objnumdistr(:,:,1));mean(objnumdistr(:,:,2))];
            A = gnf_normalize(A,2);
            b = gnf_normalize(b,2);
            alpha{1}(x,y,:) = gnf_lsqlin(A',b');
            % Approach 2: Object number fraction multinomial linear unmixing
            alpha{2}(x,y,1) = tz_fitmnom2(testobjdistr,pdf(1,:),pdf(2,:));
            alpha{2}(x,y,2) = 1 - alpha{2}(x,y,1);
            % Approach 3: Fluorescence fraction concentration estimation
            % and unmixing
            w = gnf_solvemix(A_fluo,totalfluo');
            concentration(x,y,:) = w;
            alpha{3}(x,y,:) = gnf_normalize(w'.*Asum,2)';
        end
    end
end
if ~exist('ysave','var')
    return
end
if ysave
    save(['./inter_results/block' int2str(block) '/estimation.mat'],...
        'alpha','concentration');
end