function gnf_verify(block)
% FUNCTION GNF_VERIFY plots the estimated dye concentration for both
% protein patterns versus the actual concentration for comparison
%
% Author: T. Peng
% Created: 12-Apr-2008
% Last Update: 23-Dec-2009
%
% Copyright (C) 2008-2010 Murphy Lab, Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

resultpath = ['./inter_results/block' int2str(block)];

fz_str = 6;
fz_num = 6;

load([resultpath '/estimation.mat'])
load([resultpath '/objectdistri.mat'])
ur = [700,300];

concen = concentration;
concen(1,:,:) = [];
concen(:,1,:) = [];
concen(concen<0) = NaN;

disp('Estimated vs. actual concentration correlation coefficients')

subplot(121)
UR = ur(1);
r = gnf_corrcoef(mitoconcen,concen(:,:,1)');
plot(mitoconcen,concen(:,:,1)','k^','MarkerSize',2.5,'MarkerFaceColor','k')
hold on
plot([0 UR],[0 UR],'k--')
axis equal
axis([0 UR 0 UR])
xlabel('Actual conc.','fontsize',fz_str)
ylabel('Estimated conc.','fontsize',fz_str)
set(gca,'fontsize',fz_num)
title('Mitotracker','fontsize',fz_str)
disp(['mitotracker r = ' num2str(r)])

subplot(122)
UR = ur(2);
r = gnf_corrcoef(lysoconcen,concen(:,:,2));
plot(lysoconcen,concen(:,:,2),'ks','MarkerSize',2.5,'MarkerFaceColor','k')
hold on
plot([0 UR],[0 UR],'k--')
axis equal
axis([0 UR 0 UR])
xlabel('Actual conc.','fontsize',fz_str)
ylabel('Estimated conc.','fontsize',fz_str)
set(gca,'fontsize',fz_num)
title('Lysotracker','fontsize',fz_str)
disp(['lysotracker r = ' num2str(r)])

% Total correlatoin coefficient
expConcen(:,:,1) = repmat(mitoconcen',[size(concen,1),1]);
expConcen(:,:,2) = repmat(lysoconcen,[1,size(concen,2)]);
r = gnf_corrcoef(expConcen,concen);
disp(['Overall r = ' num2str(r)])